@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: totemGMPPlugin.cpp,v 1.13 2006/10/16 19:12:23 hadess Exp $
+ * $Id: totemGMPPlugin.cpp,v 1.14 2006/10/24 22:58:50 hadess Exp $
  */
 
 #include <mozilla-config.h>
@@ -61,7 +61,8 @@ static const totemPluginMimeEntry kMimeTypes[] = {
 	{ "video/x-ms-asx", "asx", NULL, TRUE },
 	{ "audio/x-ms-asx", "asx", NULL, TRUE },
 	{ "video/x-ms-wma", "wma", NULL, TRUE },
-	{ "audio/x-ms-wax", "wax", NULL, TRUE }
+	{ "audio/x-ms-wax", "wax", NULL, TRUE },
+	{ "video/mpeg", "mpg", NULL, TRUE }
 };
 
 totemScriptablePlugin::totemScriptablePlugin (totemPlugin *aPlugin)
