@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: totemNarrowSpacePlugin.cpp,v 1.5 2006/10/01 12:05:42 hadess Exp $
+ * $Id: totemNarrowSpacePlugin.cpp,v 1.7 2006/10/08 15:13:12 hadess Exp $
  */
 
 #include <mozilla-config.h>
@@ -32,6 +32,10 @@
 #include <nsMemory.h>
 #include <nsXPCOM.h>
 
+#define GNOME_ENABLE_DEBUG 1
+/* define GNOME_ENABLE_DEBUG for more debug spew */
+#include "debug.h"
+
 #include "totemNarrowSpacePlugin.h"
 
 /* 2e390ee1-f0e3-423c-9764-f5ab50a40c06 */
@@ -43,21 +47,21 @@ static const char kClassDescription[] = "totemNarrowSpacePlugin";
 static const char kPluginDescription[] = "QuickTime Plug-in 7.0 (compatible; Totem)";
 
 static const totemPluginMimeEntry kMimeTypes[] = {
-	{ "video/quicktime", "mov", NULL },
-	{ "video/mp4", "mp4", NULL },
-	{ "image/x-macpaint", "pntg", NULL },
-	{ "image/x-quicktime", "pict, pict1, pict2", "image/x-pict" }
+	{ "video/quicktime", "mov", NULL, FALSE },
+	{ "video/mp4", "mp4", NULL, FALSE },
+	{ "image/x-macpaint", "pntg", NULL, FALSE },
+	{ "image/x-quicktime", "pict, pict1, pict2", "image/x-pict", FALSE }
 };
 
 totemScriptablePlugin::totemScriptablePlugin (totemPlugin *aPlugin)
   : mPlugin(aPlugin)
 {
-  g_print ("%s ctor [%p]\n", kClassDescription, (void*) this);
+  D ("%s ctor [%p]", kClassDescription, (void*) this);
 }
 
 totemScriptablePlugin::~totemScriptablePlugin ()
 {
-  g_print ("%s dtor [%p]\n", kClassDescription, (void*) this);
+  D ("%s dtor [%p]", kClassDescription, (void*) this);
 }
 
 /* static */ char *
@@ -175,29 +179,23 @@ totemScriptablePlugin::GetClassIDNoAlloc (nsCID *aClassIDNoAlloc)
 NS_IMETHODIMP
 totemScriptablePlugin::Play ()
 {
-  NS_ENSURE_STATE (mPlugin);
+  NS_ENSURE_STATE (IsValid ());
 
-  totem_plugin_play (mPlugin);
-
-  return NS_OK;
+  return mPlugin->Play ();
 }
 
 NS_IMETHODIMP
 totemScriptablePlugin::Rewind ()
 {
-  NS_ENSURE_STATE (mPlugin);
+  NS_ENSURE_STATE (IsValid ());
 
-  totem_plugin_pause (mPlugin);
-
-  return NS_OK;
+  return mPlugin->Pause ();
 }
 
 NS_IMETHODIMP
 totemScriptablePlugin::Stop ()
 {
-  NS_ENSURE_STATE (mPlugin);
+  NS_ENSURE_STATE (IsValid ());
 
-  totem_plugin_stop (mPlugin);
-
-  return NS_OK;
+  return mPlugin->Stop ();
 }
