@@ -80,14 +80,14 @@ totem_lang_table_parse_start_tag (GMarkupParseContext *ctx,
 			/* skip if empty */
 			if (**attr_values)
 			{
-				g_return_if_fail (strlen (*attr_values) == 3);
+				g_return_if_fail (strlen (*attr_values) == 3 || strcmp (*attr_values, "qaa-qtz") == 0);
 				ccode_longB = *attr_values;
 			}
 		} else if (g_str_equal (*attr_names, "iso_639_2T_code")) {
 			/* skip if empty */
 			if (**attr_values)
 			{
-				g_return_if_fail (strlen (*attr_values) == 3);
+				g_return_if_fail (strlen (*attr_values) == 3 || strcmp (*attr_values, "qaa-qtz") == 0);
 				ccode_longT = *attr_values;
 			}
 		} else if (g_str_equal (*attr_names, "name")) {
@@ -424,11 +424,13 @@ static void
 on_recent_file_item_activated (GtkAction *action,
                                Totem *totem)
 {
-	char *uri;
+	GtkRecentInfo *recent_info;
+	const gchar *uri;
 	gboolean playlist_changed;
 	guint end;
 
-	uri = g_object_get_data (G_OBJECT (action), "recent-uri");
+	recent_info = g_object_get_data (G_OBJECT (action), "recent-info");
+	uri = gtk_recent_info_get_uri (recent_info);
 
 	totem_signal_block_by_data (totem->playlist, totem);
 
@@ -515,12 +517,9 @@ totem_recent_manager_changed_callback (GtkRecentManager *recent_manager, Totem *
 			       "label", label,
 			       NULL);
 
-		g_object_set_data_full (G_OBJECT (action), "recent-uri",
-				g_strdup (gtk_recent_info_get_uri (info)),
-				g_free);
-		g_object_set_data_full (G_OBJECT (action), "pixbuf-icon",
-				gtk_recent_info_get_icon (info, GTK_ICON_SIZE_MENU),
-				g_object_unref);
+		g_object_set_data_full (G_OBJECT (action), "recent-info",
+				gtk_recent_info_ref (info),
+				(GDestroyNotify) gtk_recent_info_unref);
 		g_signal_connect (G_OBJECT (action), "activate",
 			  	G_CALLBACK (on_recent_file_item_activated),
 				totem);
@@ -1067,16 +1066,16 @@ static const GtkActionEntry entries[] = {
 	{ "movie-menu", NULL, N_("_Movie") },
 	{ "open", GTK_STOCK_OPEN, N_("_Open..."), "<control>O", N_("Open a file"), G_CALLBACK (open_action_callback) },
 	{ "open-location", NULL, N_("Open _Location..."), "<control>L", N_("Open a non-local file"), G_CALLBACK (open_location_action_callback) },
-	{ "eject", NULL, N_("_Eject"), "<control>E", NULL, G_CALLBACK (eject_action_callback) },
+	{ "eject", "media-eject", N_("_Eject"), "<control>E", NULL, G_CALLBACK (eject_action_callback) },
 	{ "play", GTK_STOCK_MEDIA_PLAY, N_("Play / Pa_use"), "P", N_("Play or pause the movie"), G_CALLBACK (play_action_callback) },
 	{ "quit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q", N_("Quit the program"), G_CALLBACK (quit_action_callback) },
 
 	{ "edit-menu", NULL, N_("_Edit") },
-	{ "take-screenshot", GTK_STOCK_CONVERT, N_("Take _Screenshot..."), "<control>S", N_("Take a screenshot"), G_CALLBACK (take_screenshot_action_callback) },
+	{ "take-screenshot", "camera-photo", N_("Take _Screenshot..."), "<control>S", N_("Take a screenshot"), G_CALLBACK (take_screenshot_action_callback) },
 	{ "preferences", GTK_STOCK_PREFERENCES, N_("Prefere_nces"), NULL, NULL, G_CALLBACK (preferences_action_callback) },
 
 	{ "view-menu", NULL, N_("_View") },
-	{ "fullscreen", GTK_STOCK_ZOOM_FIT, N_("_Fullscreen"), "F", N_("Switch to fullscreen"), G_CALLBACK (fullscreen_action_callback) },
+	{ "fullscreen", "view-fullscreen", N_("_Fullscreen"), "F", N_("Switch to fullscreen"), G_CALLBACK (fullscreen_action_callback) },
 	{ "zoom-window-menu", NULL, N_("Fit Window to Movie") },
 	{ "zoom-1-2", NULL, N_("_Resize 1:2"), "0", N_("Resize to half the video size"), G_CALLBACK (zoom_1_2_action_callback) },
 	{ "zoom-1-1", NULL, N_("Resize _1:1"), "1", N_("Resize to video size"), G_CALLBACK (zoom_1_1_action_callback) },
@@ -1094,13 +1093,11 @@ static const GtkActionEntry entries[] = {
 	{ "next-chapter", GTK_STOCK_MEDIA_NEXT, N_("_Next Chapter/Movie"), "n", N_("Next chapter or movie"), G_CALLBACK (next_chapter_action_callback) },
 	{ "previous-chapter", GTK_STOCK_MEDIA_PREVIOUS, N_("_Previous Chapter/Movie"), "b", N_("Previous chapter or movie"), G_CALLBACK (previous_chapter_action_callback) },
 	{ "skip-to", GTK_STOCK_JUMP_TO, N_("_Skip to..."), "S", N_("Skip to a specific time"), G_CALLBACK (skip_to_action_callback) },
-	{ "skip-forward", GTK_STOCK_MEDIA_FORWARD, N_("Skip _Forward"), "Right", N_("Skip forward"), G_CALLBACK (skip_forward_action_callback) },
-	{ "skip-backwards", GTK_STOCK_MEDIA_REWIND, N_("Skip _Backwards"), "Left", N_("Skip backwards"), G_CALLBACK (skip_backwards_action_callback) },
 
 	{ "sound-menu", NULL, N_("_Sound") },
 /*	{ "languages-menu", NULL, N_("_Languages") }, */
-	{ "volume-up", NULL, N_("Volume _Up"), "Up", N_("Volume up"), G_CALLBACK (volume_up_action_callback) },
-	{ "volume-down", NULL, N_("Volume _Down"), "Down", N_("Volume down"), G_CALLBACK (volume_down_action_callback) },
+	{ "volume-up", "audio-volume-high", N_("Volume _Up"), "Up", N_("Volume up"), G_CALLBACK (volume_up_action_callback) },
+	{ "volume-down", "audio-volume-low", N_("Volume _Down"), "Down", N_("Volume down"), G_CALLBACK (volume_down_action_callback) },
 
 	{ "help-menu", NULL, N_("_Help") },
 	{ "contents", GTK_STOCK_HELP, N_("_Contents"), "F1", N_("Help contents"), G_CALLBACK (contents_action_callback) },
@@ -1113,6 +1110,16 @@ static const GtkActionEntry zoom_entries[] = {
 	{ "zoom-out", GTK_STOCK_ZOOM_OUT, N_("Zoom Out"), "T", N_("Zoom out"), G_CALLBACK (zoom_out_action_callback) }
 };
 
+static const GtkActionEntry seek_entries_ltr[] = {
+	{ "skip-forward", GTK_STOCK_MEDIA_FORWARD, N_("Skip _Forward"), "Right", N_("Skip forward"), G_CALLBACK (skip_forward_action_callback) },
+	{ "skip-backwards", GTK_STOCK_MEDIA_REWIND, N_("Skip _Backwards"), "Left", N_("Skip backwards"), G_CALLBACK (skip_backwards_action_callback) }
+};
+
+static const GtkActionEntry seek_entries_rtl[] = {
+	{ "skip-forward", GTK_STOCK_MEDIA_FORWARD, N_("Skip _Forward"), "Left", N_("Skip forward"), G_CALLBACK (skip_forward_action_callback) },
+	{ "skip-backwards", GTK_STOCK_MEDIA_REWIND, N_("Skip _Backwards"), "Right", N_("Skip backwards"), G_CALLBACK (skip_backwards_action_callback) }
+};
+
 static const GtkToggleActionEntry toggle_entries[] = {
 	{ "repeat-mode", NULL, N_("_Repeat Mode"), NULL, N_("Set the repeat mode"), G_CALLBACK (repeat_mode_action_callback), FALSE },
 	{ "shuffle-mode", NULL, N_("Shuff_le Mode"), NULL, N_("Set the shuffle mode"), G_CALLBACK (shuffle_mode_action_callback), FALSE },
@@ -1134,14 +1141,26 @@ static void
 totem_ui_manager_connect_proxy_callback (GtkUIManager *ui_manager,
 		GtkAction *action, GtkWidget *widget, Totem *totem)
 {
+	GtkRecentInfo *recent_info;
 	GdkPixbuf *icon;
 	GtkWidget *image;
+	gint w, h;
 
-	icon = g_object_get_data (G_OBJECT (action), "pixbuf-icon");
-	if (icon != NULL && GTK_IS_IMAGE_MENU_ITEM (widget)) {
-		image = gtk_image_new_from_pixbuf (icon);
-		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (widget),
-				image);
+	recent_info = g_object_get_data (G_OBJECT (action), "recent-info");
+
+	if (recent_info == NULL) {
+		return;
+	}
+
+	if (GTK_IS_IMAGE_MENU_ITEM (widget)) {
+		gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &w, &h);
+		icon = gtk_recent_info_get_icon (recent_info, w);
+
+		if (icon != NULL) {
+			image = gtk_image_new_from_pixbuf (icon);
+			gtk_image_menu_item_set_image
+				(GTK_IMAGE_MENU_ITEM (widget), image);
+		}
 	}
 }
 
@@ -1154,12 +1173,22 @@ totem_ui_manager_setup (Totem *totem)
 	GtkAction *action;
 
 	totem->main_action_group = gtk_action_group_new ("main-action-group");
-	gtk_action_group_set_translation_domain (totem->main_action_group, GETTEXT_PACKAGE);
+	gtk_action_group_set_translation_domain (totem->main_action_group,
+			GETTEXT_PACKAGE);
 	gtk_action_group_add_actions (totem->main_action_group, entries,
 		G_N_ELEMENTS (entries), totem);
+	if (gtk_widget_get_direction (totem->win) == GTK_TEXT_DIR_RTL) {
+		gtk_action_group_add_actions (totem->main_action_group,
+				seek_entries_rtl,
+				G_N_ELEMENTS (seek_entries_rtl), totem);
+	} else {
+		gtk_action_group_add_actions (totem->main_action_group,
+				seek_entries_ltr,
+				G_N_ELEMENTS (seek_entries_ltr), totem);
+	}
 	gtk_action_group_add_toggle_actions (totem->main_action_group, 
 		toggle_entries, G_N_ELEMENTS (toggle_entries), totem);
-	gtk_action_group_add_radio_actions (totem->main_action_group, 
+	gtk_action_group_add_radio_actions (totem->main_action_group,
 		aspect_ratio_entries, G_N_ELEMENTS (aspect_ratio_entries), 0,
 		G_CALLBACK (aspect_ratio_changed_callback), totem);
 
@@ -1177,30 +1206,35 @@ totem_ui_manager_setup (Totem *totem)
 	g_object_unref (action);
 
 	totem->zoom_action_group = gtk_action_group_new ("zoom-action-group");
-	gtk_action_group_set_translation_domain (totem->zoom_action_group, GETTEXT_PACKAGE);
+	gtk_action_group_set_translation_domain (totem->zoom_action_group,
+			GETTEXT_PACKAGE);
 	gtk_action_group_add_actions (totem->zoom_action_group, zoom_entries,
 		G_N_ELEMENTS (zoom_entries), totem);
 
 	totem->ui_manager = gtk_ui_manager_new ();
 	g_signal_connect (G_OBJECT (totem->ui_manager), "connect-proxy",
-			G_CALLBACK (totem_ui_manager_connect_proxy_callback), totem);
+			G_CALLBACK (totem_ui_manager_connect_proxy_callback),
+			totem);
 	gtk_ui_manager_insert_action_group (totem->ui_manager,
-		totem->main_action_group, 0);
+			totem->main_action_group, 0);
 	gtk_ui_manager_insert_action_group (totem->ui_manager,
-		totem->zoom_action_group, -1);
+			totem->zoom_action_group, -1);
 
 	totem->devices_action_group = NULL;
 	totem->devices_ui_id = gtk_ui_manager_new_merge_id (totem->ui_manager);
 	totem->languages_action_group = NULL;
-	totem->languages_ui_id = gtk_ui_manager_new_merge_id (totem->ui_manager);
+	totem->languages_ui_id = gtk_ui_manager_new_merge_id
+		(totem->ui_manager);
 	totem->subtitles_action_group = NULL;
-	totem->subtitles_ui_id = gtk_ui_manager_new_merge_id (totem->ui_manager);
+	totem->subtitles_ui_id = gtk_ui_manager_new_merge_id
+		(totem->ui_manager);
 
 	gtk_window_add_accel_group (GTK_WINDOW (totem->win),
-		gtk_ui_manager_get_accel_group (totem->ui_manager));
+			gtk_ui_manager_get_accel_group (totem->ui_manager));
 
 	filename = totem_interface_get_full_path ("totem-ui.xml");
-	if (!gtk_ui_manager_add_ui_from_file (totem->ui_manager, filename, NULL)) {
+	if (gtk_ui_manager_add_ui_from_file (totem->ui_manager,
+				filename, NULL) == 0) {
 		totem_interface_error_blocking (
 			_("Couldn't load the 'ui description' file"),
 			_("Make sure that Totem is properly installed."),
