@@ -26,6 +26,8 @@
 #include <gtk/gtkvbox.h>
 #include <libgnomevfs/gnome-vfs-volume.h>
 
+#include "totem-pl-parser.h"
+
 G_BEGIN_DECLS
 
 #define GTK_TYPE_PLAYLIST            (totem_playlist_get_type ())
@@ -72,6 +74,8 @@ gboolean totem_playlist_add_mrl  (TotemPlaylist *playlist, const char *mrl,
 
 void totem_playlist_save_current_playlist (TotemPlaylist *playlist,
 					   const char *output);
+void totem_playlist_save_current_playlist_ext (TotemPlaylist *playlist,
+					   const char *output, TotemPlParserType type);
 
 /* totem_playlist_clear doesn't emit the current_removed signal, even if it does
  * because the caller should know what to do after it's done with clearing */
@@ -88,7 +92,8 @@ gboolean   totem_playlist_get_current_metadata (TotemPlaylist *playlist,
 						char         **title,
 						char         **album);
 gboolean   totem_playlist_set_title (TotemPlaylist *playlist,
-				     const char *title);
+				     const char *title,
+				     gboolean force);
 
 #define    totem_playlist_has_direction(playlist, direction) (direction == TOTEM_PLAYLIST_DIRECTION_NEXT ? totem_playlist_has_next_mrl (playlist) : totem_playlist_has_previous_mrl (playlist))
 gboolean   totem_playlist_has_previous_mrl (TotemPlaylist *playlist);
