@@ -66,7 +66,6 @@
 #include "totem.h"
 #include "totem-private.h"
 #include "totem-preferences.h"
-#include "totem-stock-icons.h"
 #include "totem-disc.h"
 
 #include "debug.h"
@@ -230,7 +229,6 @@ totem_action_exit (Totem *totem)
 
 	totem_sublang_exit (totem);
 	totem_destroy_file_filters ();
-	totem_named_icons_dispose (totem);
 
 	if (totem->gc)
 		g_object_unref (G_OBJECT (totem->gc));
@@ -268,6 +266,7 @@ play_pause_set_label (Totem *totem, TotemStates state)
 {
 	GtkAction *action;
 	const char *id, *tip;
+	GSList *l, *proxies;
 
 	if (state == totem->state)
 		return;
@@ -303,6 +302,12 @@ play_pause_set_label (Totem *totem, TotemStates state)
 			"tooltip", _(tip),
 			"stock-id", id, NULL);
 
+	proxies = gtk_action_get_proxies (action);
+	for (l = proxies; l != NULL; l = l->next) {
+		atk_object_set_name (gtk_widget_get_accessible (l->data),
+				_(tip));
+	}
+
 	totem->state = state;
 }
 
@@ -592,6 +597,7 @@ window_state_event_cb (GtkWidget *window, GdkEventWindowState *event,
 
 		totem->controls_visibility = TOTEM_CONTROLS_FULLSCREEN;
 		show_controls (totem, FALSE);
+		totem_action_set_sensitivity ("fullscreen", FALSE);
 	} else {
 		GtkAction *action;
 
@@ -602,16 +608,15 @@ window_state_event_cb (GtkWidget *window, GdkEventWindowState *event,
 		totem_scrsaver_enable (totem->scr);
 
 		action = gtk_action_group_get_action (totem->main_action_group,
-			"show-controls");
+				"show-controls");
 
 		if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)))
-		{
 			totem->controls_visibility = TOTEM_CONTROLS_VISIBLE;
-			show_controls (totem, TRUE);
-		} else {
+		else
 			totem->controls_visibility = TOTEM_CONTROLS_HIDDEN;
-			show_controls (totem, TRUE);
-		}
+
+		show_controls (totem, TRUE);
+		totem_action_set_sensitivity ("fullscreen", TRUE);
 	}
 
 	return FALSE;
@@ -1401,12 +1406,6 @@ on_error_event (BaconVideoWidget *bvw, char *message,
 	}
 }
 
-static void
-on_speed_warning_event (BaconVideoWidget *bvw, Totem *totem)
-{
-	g_message ("TBD: Implement speed warning");
-}
-
 static void
 on_buffering_event (BaconVideoWidget *bvw, int percentage, Totem *totem)
 {
@@ -1533,16 +1532,6 @@ seek_slider_pressed_cb (GtkWidget *widget, GdkEventButton *event, Totem *totem)
 	totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), TRUE);
 	totem_time_label_set_seeking (TOTEM_TIME_LABEL (totem->tcw_time_label), TRUE);
 
-	if (bacon_video_widget_can_direct_seek (totem->bvw) != FALSE)
-	{
-		if (!totem->seek_in_progress) {
-			totem->was_playing = bacon_video_widget_is_playing (totem->bvw);
-			bacon_video_widget_pause (totem->bvw);
-			/* We need to remember that we are already paused and seeking */
-			totem->seek_in_progress = TRUE;
-		}
-	}
-
 	return FALSE;
 }
 
@@ -1584,12 +1573,8 @@ seek_slider_released_cb (GtkWidget *widget, GdkEventButton *event, Totem *totem)
 	val = gtk_adjustment_get_value (adj);
 	gtk_adjustment_set_value (other_adj, val);
 
-	if (bacon_video_widget_can_direct_seek (totem->bvw) != FALSE)
+	if (bacon_video_widget_can_direct_seek (totem->bvw) == FALSE)
 	{
-		if (totem->was_playing)
-			bacon_video_widget_play (totem->bvw, NULL);
-		totem->seek_in_progress = FALSE;
-	} else {
 		totem_action_seek (totem, val / 65535.0);
 	}
 
@@ -2005,14 +1990,14 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 	case TOTEM_REMOTE_COMMAND_TOGGLE_CONTROLS:
 		if (totem->controls_visibility != TOTEM_CONTROLS_FULLSCREEN)
 		{
-			GtkCheckMenuItem *item;
-			gboolean value;
+			GtkToggleAction *action;
+			gboolean state;
 
-			item = GTK_CHECK_MENU_ITEM (glade_xml_get_widget
-					(totem->xml,
-					 "tmw_show_controls_menu_item"));
-			value = gtk_check_menu_item_get_active (item);
-			gtk_check_menu_item_set_active (item, !value);
+			action = GTK_TOGGLE_ACTION (gtk_action_group_get_action
+					(totem->main_action_group,
+					 "show-controls"));
+			state = gtk_toggle_action_get_active (action);
+			gtk_toggle_action_set_active (action, !state);
 		}
 		break;
 	case TOTEM_REMOTE_COMMAND_SHOW_PLAYING:
@@ -2235,6 +2220,18 @@ theme_changed_cb (GtkIconTheme *icon_theme, Totem *totem)
 	move_popups (totem);
 }
 
+static void
+window_realize_cb (GtkWidget *widget, Totem *totem)
+{
+	GdkScreen *screen;
+
+	screen = gtk_widget_get_screen (widget);
+	g_signal_connect (G_OBJECT (screen), "size-changed",
+			G_CALLBACK (size_changed_cb), totem);
+	g_signal_connect (G_OBJECT (gtk_icon_theme_get_for_screen (screen)),
+			"changed", G_CALLBACK (theme_changed_cb), totem);
+}
+
 static void
 popup_timeout_add(Totem* totem)
 {
@@ -2355,29 +2352,57 @@ totem_action_handle_key_release (Totem *totem, GdkEventKey *event)
 {
 	gboolean retval = TRUE;
 
-	/* Alphabetical */
 	switch (event->keyval) {
 	case GDK_Left:
 	case GDK_Right:
-		totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), FALSE);
-		if (bacon_video_widget_can_direct_seek (totem->bvw)) {
-			if (totem->was_playing) {
-				bacon_video_widget_play (totem->bvw, NULL);
-			}
-			totem->seek_in_progress = FALSE;
-		}
+		totem_statusbar_set_seeking
+			(TOTEM_STATUSBAR (totem->statusbar), FALSE);
 		break;
 	}
 
 	return retval;
 }
 
+static void
+totem_action_handle_seek (Totem *totem, GdkEventKey *event, gboolean is_forward)
+{
+	if (is_forward != FALSE) {
+		if (event->state & GDK_SHIFT_MASK) {
+			totem_action_seek_relative (totem,
+					SEEK_FORWARD_SHORT_OFFSET);
+		} else if (event->state & GDK_CONTROL_MASK) {
+			totem_action_seek_relative (totem,
+					SEEK_FORWARD_LONG_OFFSET);
+		} else {
+			totem_action_seek_relative (totem,
+					SEEK_FORWARD_OFFSET);
+		}
+	} else {
+		if (event->state & GDK_SHIFT_MASK) {
+			totem_action_seek_relative (totem,
+					SEEK_BACKWARD_SHORT_OFFSET);
+		} else if (event->state & GDK_CONTROL_MASK) {
+			totem_action_seek_relative (totem,
+					SEEK_BACKWARD_LONG_OFFSET);
+		} else {
+			totem_action_seek_relative (totem,
+					SEEK_BACKWARD_OFFSET);
+		}
+	}
+}
+
 static gboolean
 totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 {
-	gboolean retval = TRUE;
+	gboolean retval = TRUE, playlist_focused = FALSE;
+	GtkWidget *focused;
+
+	focused = gtk_window_get_focus (GTK_WINDOW (totem->win));
+	if (focused != NULL && gtk_widget_is_ancestor
+			(focused, GTK_WIDGET (totem->playlist)) != FALSE) {
+		playlist_focused = TRUE;
+	}
 
-	/* Alphabetical */
 	switch (event->keyval) {
 	case GDK_A:
 	case GDK_a:
@@ -2424,14 +2449,14 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 	case GDK_i:
 	case GDK_I:
 		{
-			GtkCheckMenuItem *item;
-			gboolean value;
+			GtkToggleAction *action;
+			gboolean state;
 
-			item = GTK_CHECK_MENU_ITEM (glade_xml_get_widget
-					(totem->xml,
-					 "tmw_deinterlace_menu_item"));
-			value = gtk_check_menu_item_get_active (item);
-			gtk_check_menu_item_set_active (item, !value);
+			action = GTK_TOGGLE_ACTION (gtk_action_group_get_action
+					(totem->main_action_group,
+					 "deinterlace"));
+			state = gtk_toggle_action_get_active (action);
+			gtk_toggle_action_set_active (action, !state);
 		}
 		break;
 	case GDK_M:
@@ -2485,46 +2510,26 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 			totem_action_fullscreen (totem, FALSE);
 		break;
 	case GDK_Left:
-		totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), TRUE);
-		if (bacon_video_widget_can_direct_seek (totem->bvw)) {
-			if (!totem->seek_in_progress) {
-				totem->was_playing = bacon_video_widget_is_playing (totem->bvw);
-				bacon_video_widget_pause (totem->bvw);
-				/* We need to remember that we are already paused and seeking */
-				totem->seek_in_progress = TRUE;
-			}
-		}
-		if (event->state & GDK_SHIFT_MASK) {
-			totem_action_seek_relative (totem,
-					SEEK_BACKWARD_SHORT_OFFSET);
-		} else if (event->state & GDK_CONTROL_MASK) {
-			totem_action_seek_relative (totem,
-					SEEK_BACKWARD_LONG_OFFSET);
-		} else {
-			totem_action_seek_relative (totem,
-					SEEK_BACKWARD_OFFSET);
-		}
+		if (playlist_focused != FALSE)
+			return FALSE;
+
+		totem_statusbar_set_seeking
+			(TOTEM_STATUSBAR (totem->statusbar), TRUE);
+		if (gtk_widget_get_direction (totem->win) == GTK_TEXT_DIR_RTL)
+			totem_action_handle_seek (totem, event, TRUE);
+		else
+			totem_action_handle_seek (totem, event, FALSE);
 		break;
 	case GDK_Right:
-		totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), TRUE);
-		if (bacon_video_widget_can_direct_seek (totem->bvw)) {
-			if (!totem->seek_in_progress) {
-				totem->was_playing = bacon_video_widget_is_playing (totem->bvw);
-				bacon_video_widget_pause (totem->bvw);
-				/* We need to remember that we are already paused and seeking */
-				totem->seek_in_progress = TRUE;
-			}
-		}
-		if (event->state & GDK_SHIFT_MASK) {
-			totem_action_seek_relative (totem,
-					SEEK_FORWARD_SHORT_OFFSET);
-		} else if (event->state & GDK_CONTROL_MASK) {
-			totem_action_seek_relative (totem,
-					SEEK_FORWARD_LONG_OFFSET);
-		} else {
-			totem_action_seek_relative (totem,
-					SEEK_FORWARD_OFFSET);
-		}
+		if (playlist_focused != FALSE)
+			return FALSE;
+
+		totem_statusbar_set_seeking
+			(TOTEM_STATUSBAR (totem->statusbar), TRUE);
+		if (gtk_widget_get_direction (totem->win) == GTK_TEXT_DIR_RTL)
+			totem_action_handle_seek (totem, event, FALSE);
+		else
+			totem_action_handle_seek (totem, event, TRUE);
 		break;
 	case GDK_space:
 		if (totem_is_fullscreen (totem) != FALSE || gtk_widget_is_focus (GTK_WIDGET (totem->bvw)) != FALSE)
@@ -2533,9 +2538,13 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 			retval = FALSE;
 		break;
 	case GDK_Up:
+		if (playlist_focused != FALSE)
+			return FALSE;
 		totem_action_volume_relative (totem, VOLUME_UP_OFFSET);
 		break;
 	case GDK_Down:
+		if (playlist_focused != FALSE)
+			return FALSE;
 		totem_action_volume_relative (totem, VOLUME_DOWN_OFFSET);
 		break;
 	case GDK_0:
@@ -2555,18 +2564,15 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 		totem_action_set_scale_ratio (totem, 2);
 		break;
 	case GDK_Menu:
+		if (playlist_focused != FALSE)
+			return FALSE;
+		totem_action_menu_popup (totem, 0);
+		break;
 	case GDK_F10:
-		/* Chain up if the playlist has the focuse */
-		{
-			GtkWidget *focused;
-
-			if (event->keyval != GDK_Menu && !(event->state & GDK_SHIFT_MASK))
-				return FALSE;
-
-			focused = gtk_window_get_focus (GTK_WINDOW (totem->win));
-			if (focused != NULL && gtk_widget_is_ancestor (focused, GTK_WIDGET (totem->playlist)) != FALSE)
-				return FALSE;
-		}
+		if (playlist_focused != FALSE)
+			return FALSE;
+		if (!(event->state & GDK_SHIFT_MASK))
+			return FALSE;
 
 		totem_action_menu_popup (totem, 0);
 		break;
@@ -2884,16 +2890,24 @@ totem_callback_connect (Totem *totem)
 	/* Controls */
 	box = glade_xml_get_widget (totem->xml, "tmw_buttons_hbox");
 
-	action = gtk_action_group_get_action (totem->main_action_group, "previous-chapter");
+	action = gtk_action_group_get_action (totem->main_action_group,
+			"previous-chapter");
 	item = gtk_action_create_tool_item (action);
+	atk_object_set_name (gtk_widget_get_accessible (item),
+			_("Previous Chapter/Movie"));
 	gtk_box_pack_start (GTK_BOX (box), item, FALSE, FALSE, 0);
 
 	action = gtk_action_group_get_action (totem->main_action_group, "play");
 	item = gtk_action_create_tool_item (action);
+	atk_object_set_name (gtk_widget_get_accessible (item),
+			_("Play / Pause"));
 	gtk_box_pack_start (GTK_BOX (box), item, FALSE, FALSE, 0);
 
-	action = gtk_action_group_get_action (totem->main_action_group, "next-chapter");
+	action = gtk_action_group_get_action (totem->main_action_group,
+			"next-chapter");
 	item = gtk_action_create_tool_item (action);
+	atk_object_set_name (gtk_widget_get_accessible (item),
+			_("Next Chapter/Movie"));
 	gtk_box_pack_start (GTK_BOX (box), item, FALSE, FALSE, 0);
 
 	/* Sidebar button (Drag'n'Drop) */
@@ -2921,10 +2935,8 @@ totem_callback_connect (Totem *totem)
 			G_CALLBACK (window_state_event_cb), totem);
 
 	/* Screen size and Theme changes */
-	g_signal_connect (G_OBJECT (gdk_screen_get_default ()),
-			"size-changed", G_CALLBACK (size_changed_cb), totem);
-	g_signal_connect (G_OBJECT (gtk_icon_theme_get_default ()),
-			"changed", G_CALLBACK (theme_changed_cb), totem);
+	g_signal_connect (totem->win, "realize",
+			G_CALLBACK (window_realize_cb), totem);
 
 	/* Motion notify for the Popups */
 	item = glade_xml_get_widget (totem->xml,
@@ -3021,9 +3033,6 @@ totem_callback_connect (Totem *totem)
 	g_signal_connect (G_OBJECT (totem->volume), "value-changed",
 			G_CALLBACK (vol_cb), totem);
 
-	/* Named icon support */
-	totem_set_default_icons (totem);
-
 	/* Set sensitivity of the toolbar buttons */
 	totem_action_set_sensitivity ("play", FALSE);
 	totem_action_set_sensitivity ("next-chapter", FALSE);
@@ -3130,10 +3139,6 @@ video_widget_create (Totem *totem)
 			"error",
 			G_CALLBACK (on_error_event),
 			totem);
-	g_signal_connect (G_OBJECT (totem->bvw),
-			"speed-warning",
-			G_CALLBACK (on_speed_warning_event),
-			totem);
 
 	container = glade_xml_get_widget (totem->xml, "tmw_bvw_vbox");
 	gtk_container_add (GTK_CONTAINER (container),
@@ -3285,8 +3290,6 @@ main (int argc, char **argv)
 
 	totem->win = glade_xml_get_widget (totem->xml, "totem_main_window");
 	
-	totem_named_icons_init (totem, FALSE);
-
 	/* Menubar */
 	totem_ui_manager_setup (totem);
 
